#ifndef _sw_corby_buffer_h
#define _sw_corby_buffer_h

/*
 * Copyright 2003 Swampwolf Technologies, Inc. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY SWAMPWOLF TECHNOLOGIES, INC. ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Swampwolf Technologies, Inc.
 */

#include <salt/salt.h>
#include <salt/vtypes.h>
#include <corby/corby.h>
#include <corby/dll.h>


#ifdef __cplusplus
extern "C"
{
#endif 


struct										_sw_corby_buffer;
typedef struct _sw_corby_buffer	*	sw_corby_buffer;
struct										_sw_corby_object;


typedef enum _sw_corby_buffer_copy_mode
{
   SW_CORBY_BUFFER_SHALLOW,
   SW_CORBY_BUFFER_DEEP
} sw_corby_buffer_copy_mode;


sw_result CORBY_DLL
sw_corby_buffer_init(
				sw_corby_buffer	*	self);


sw_result CORBY_DLL
sw_corby_buffer_init_with_size(
				sw_corby_buffer	*	self,
            sw_size					size);


sw_result CORBY_DLL
sw_corby_buffer_fina(
				sw_corby_buffer	self);


void CORBY_DLL
sw_corby_buffer_reset(
				sw_corby_buffer	self);
				

sw_result CORBY_DLL
sw_corby_buffer_set_octets(
				sw_corby_buffer	self,
				sw_octets			octets,
				sw_size				size);

				
sw_octets CORBY_DLL
sw_corby_buffer_octets(
				sw_corby_buffer	self);
				
				
sw_size CORBY_DLL
sw_corby_buffer_bytes_used(
				sw_corby_buffer	self);

				
sw_size CORBY_DLL
sw_corby_buffer_size(
				sw_corby_buffer	self);
				

sw_result CORBY_DLL
sw_corby_buffer_put_octet(
				sw_corby_buffer	self,
				sw_octet				val);


sw_result CORBY_DLL
sw_corby_buffer_put_char(
				sw_corby_buffer	self,
				sw_char				val);


sw_result CORBY_DLL
sw_corby_buffer_put_ushort(
				sw_corby_buffer	self,
				sw_ushort			val);


sw_result CORBY_DLL
sw_corby_buffer_put_short(
				sw_corby_buffer	self,
				sw_short				val);


sw_result CORBY_DLL
sw_corby_buffer_put_ulong(
				sw_corby_buffer	self,
				sw_ulong				val);


sw_result CORBY_DLL
sw_corby_buffer_put_long(
				sw_corby_buffer	self,
				sw_long				val);


sw_result CORBY_DLL
sw_corby_buffer_put_buf(
				sw_corby_buffer	self,
				sw_const_octets	val,
				sw_ulong				len);
                                    

sw_result CORBY_DLL
sw_corby_buffer_put_seq(
				sw_corby_buffer	self,
				sw_const_octets	val,
				sw_ulong				len);
                                    

sw_result CORBY_DLL
sw_corby_buffer_put_cstring(
				sw_corby_buffer	self,
				sw_const_string	val);


sw_result CORBY_DLL
sw_corby_buffer_put_object(
				sw_corby_buffer						self,
				const struct _sw_corby_object	*	object);
				

sw_result CORBY_DLL
sw_corby_buffer_get_octet(
				sw_corby_buffer	self,
				sw_octet			*	val);


sw_result CORBY_DLL
sw_corby_buffer_get_char(
				sw_corby_buffer	self,
				sw_char			*	val);


sw_result CORBY_DLL
sw_corby_buffer_get_ushort(
				sw_corby_buffer	self,
				sw_ushort		*	val,
            sw_octet				endian);


sw_result CORBY_DLL
sw_corby_buffer_get_short(
				sw_corby_buffer	self,
				sw_short			*	val,
            sw_octet				endian);


sw_result CORBY_DLL
sw_corby_buffer_get_ulong(
				sw_corby_buffer	self,
				sw_ulong			*	val,
            sw_octet				endian);


sw_result CORBY_DLL
sw_corby_buffer_get_long(
				sw_corby_buffer	self,
				sw_long			*	val,
            sw_octet				endian);


sw_result CORBY_DLL
sw_corby_buffer_get_buf(
				sw_corby_buffer	self,
				sw_octets			buf,
				sw_ulong				len);


sw_result CORBY_DLL
sw_corby_buffer_get_seq(
				sw_corby_buffer				self,
				sw_octets					*	val,
				sw_ulong						*	len,
				sw_corby_buffer_copy_mode	mode,
            sw_octet							endian);
                                    

sw_result CORBY_DLL
sw_corby_buffer_get_cstring(
				sw_corby_buffer				self,
				sw_string					*	val,
				sw_ulong						*	len,
				sw_corby_buffer_copy_mode	mode,
				sw_octet							endian);
				
				
sw_result CORBY_DLL
sw_corby_buffer_get_object(
				sw_corby_buffer				self,
				struct _sw_corby_object	**	object,
				sw_octet							endian);
                                    

#ifdef __cplusplus
}
#endif 


#endif
