#ifndef _rendezvous_rendezvous_h
#define _rendezvous_rendezvous_h

/*
 * Copyright 2003 Swampwolf Technologies, Inc. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY SWAMPWOLF TECHNOLOGIES, INC. ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Swampwolf Technologies, Inc.
 */

#include <rendezvous/dll.h>
#include <salt/salt.h>
#include <salt/socket.h>
#include <salt/vtypes.h>

#if defined(__cplusplus)
extern "C"
{
#endif


struct										_sw_rendezvous;
typedef struct _sw_rendezvous		*	sw_rendezvous;


/*
 * keeps track of different rendezvous operations
 */
typedef sw_ulong							sw_rendezvous_publish_id;
typedef sw_ulong							sw_rendezvous_browse_id;
typedef sw_ulong							sw_rendezvous_resolve_id;


/*
 * status for asynchronous registration call
 */
typedef enum sw_rendezvous_publish_status
{
	SW_RENDEZVOUS_PUBLISH_STARTED,
	SW_RENDEZVOUS_PUBLISH_STOPPED,
	SW_RENDEZVOUS_PUBLISH_NAME_COLLISION,
	SW_RENDEZVOUS_PUBLISH_INVALID
} sw_rendezvous_publish_status;


typedef enum sw_rendezvous_browse_status
{
	SW_RENDEZVOUS_BROWSE_INVALID,
	SW_RENDEZVOUS_BROWSE_RELEASE,
	SW_RENDEZVOUS_BROWSE_ADD_DOMAIN,
	SW_RENDEZVOUS_BROWSE_ADD_DEFAULT_DOMAIN,
	SW_RENDEZVOUS_BROWSE_REMOVE_DOMAIN,
	SW_RENDEZVOUS_BROWSE_ADD_SERVICE,
	SW_RENDEZVOUS_BROWSE_REMOVE_SERVICE,
	SW_RENDEZVOUS_BROWSE_RESOLVED
} sw_rendezvous_browse_status;


typedef sw_opaque		sw_rendezvous_publish_handler;
typedef sw_result
(*sw_rendezvous_publish_reply)(
							sw_rendezvous_publish_handler	handler,
							sw_rendezvous						rendezvous,
							sw_rendezvous_publish_status	status,
							sw_rendezvous_publish_id		id,
							sw_opaque							extra);

typedef sw_opaque		sw_rendezvous_browse_handler;
typedef sw_result	
(*sw_rendezvous_browse_reply)(
							sw_rendezvous_browse_handler	handler,
							sw_rendezvous						rendezvous,
							sw_rendezvous_browse_id			id,
							sw_rendezvous_browse_status	status,
							sw_const_string					name,
							sw_const_string					type,
							sw_const_string					domain,
							sw_opaque							extra);

typedef sw_opaque		sw_rendezvous_resolve_handler;
typedef sw_result
(*sw_rendezvous_resolve_reply)(
							sw_rendezvous_resolve_handler	handler,
							sw_rendezvous						rendezvous,
							sw_rendezvous_resolve_id		id,
							sw_const_string					name,
							sw_const_string					type,
							sw_const_string					domain,
							sw_address							address,
							sw_port								port,
							sw_const_string					textRecord,
							sw_opaque							extra);


/*
 * API for publishing services
 */
sw_result RENDEZVOUS_DLL
sw_rendezvous_init(
						sw_rendezvous	*	self);


sw_result RENDEZVOUS_DLL
sw_rendezvous_init_with_salt(
						sw_rendezvous	*	self,
						sw_salt				salt);


sw_result RENDEZVOUS_DLL
sw_rendezvous_fina(
						sw_rendezvous		self);


sw_result RENDEZVOUS_DLL
sw_rendezvous_publish(
						sw_rendezvous						self,
						sw_const_string					name,
						sw_const_string					type,
						sw_const_string					domain,
						sw_port								port,
						sw_const_string					attrs,
						sw_rendezvous_publish_handler	handler,
						sw_rendezvous_publish_reply	reply,
						sw_opaque							extra,
						sw_rendezvous_publish_id	*	id);


sw_result RENDEZVOUS_DLL
sw_rendezvous_stop_publish(
						sw_rendezvous						self,
						sw_rendezvous_publish_id		id);


/*
 * API for browsing domains
 */
sw_result RENDEZVOUS_DLL
sw_rendezvous_browse_domains(
						sw_rendezvous						self,
						sw_rendezvous_browse_handler	handler,
						sw_rendezvous_browse_reply		reply,
						sw_opaque							extra,
						sw_rendezvous_browse_id		*	id);


sw_result RENDEZVOUS_DLL
sw_rendezvous_stop_browse_domains(
						sw_rendezvous						self,
						sw_rendezvous_browse_id			id);
						

/*
 * API for browsing services
 */
sw_result RENDEZVOUS_DLL
sw_rendezvous_browse_services(
						sw_rendezvous						self,
						sw_const_string					type,
						sw_const_string					domain,
						sw_rendezvous_browse_handler	handler,
						sw_rendezvous_browse_reply		reply,
						sw_opaque							extra,
						sw_rendezvous_browse_id		*	id);


sw_result RENDEZVOUS_DLL
sw_rendezvous_stop_browse_services(
						sw_rendezvous						self,
						sw_rendezvous_browse_id			id);



/*
 * API for resolving services
 */
sw_result RENDEZVOUS_DLL
sw_rendezvous_resolve(
						sw_rendezvous						self,
						sw_const_string					name,
						sw_const_string					type,
						sw_const_string					domain,
						sw_rendezvous_resolve_handler	handler,
						sw_rendezvous_resolve_reply	reply,
						sw_opaque							extra,
						sw_rendezvous_resolve_id	*	id);


sw_result RENDEZVOUS_DLL
sw_rendezvous_stop_resolve(
						sw_rendezvous						self,
						sw_rendezvous_resolve_id		id);
						
						
sw_result RENDEZVOUS_DLL
sw_rendezvous_run(
						sw_rendezvous						self);
						

#define SW_RENDEZVOUS_E_BASE						900
#define SW_RENDEZVOUS_E_UNKNOWN					(SW_RENDEZVOUS_E_BASE + 2)
#define SW_RENDEZVOUS_E_NO_SUCH_NAME			(SW_RENDEZVOUS_E_BASE + 3)
#define SW_RENDEZVOUS_E_NO_MEM					(SW_RENDEZVOUS_E_BASE + 4)
#define SW_RENDEZVOUS_E_BAD_PARAM				(SW_RENDEZVOUS_E_BASE + 5)
#define SW_RENDEZVOUS_E_BAD_REFERENCE			(SW_RENDEZVOUS_E_BASE + 6)
#define SW_RENDEZVOUS_E_BAD_STATE				(SW_RENDEZVOUS_E_BASE + 7)
#define SW_RENDEZVOUS_E_BAD_FLAGS				(SW_RENDEZVOUS_E_BASE + 8)
#define SW_RENDEZVOUS_E_NOT_SUPPORTED			(SW_RENDEZVOUS_E_BASE + 9)
#define SW_RENDEZVOUS_E_NOT_INITIALIZED		(SW_RENDEZVOUS_E_BASE + 10)
#define SW_RENDEZVOUS_E_NO_CACHE					(SW_RENDEZVOUS_E_BASE + 11)
#define SW_RENDEZVOUS_E_ALREADY_REGISTERED	(SW_RENDEZVOUS_E_BASE + 12)
#define SW_RENDEZVOUS_E_NAME_CONFLICT			(SW_RENDEZVOUS_E_BASE + 13)
#define SW_RENDEZVOUS_E_INVALID					(SW_RENDEZVOUS_E_BASE + 14)


#if defined(__cplusplus)
}
#endif


#endif
