#ifndef _sw_salt_log_h
#define _sw_salt_log_h

/*
 * Copyright 2003 Swampwolf Technologies, Inc. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY SWAMPWOLF TECHNOLOGIES, INC. ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Swampwolf Technologies, Inc.
 */

#include <salt/vtypes.h>
#include <salt/dll.h>


#ifdef __cplusplus
extern "C"
{
#endif 

/*
 * log is used for logging messages that cannot be turned off.  Don't call _sw_log_header or _sw_log directly.
 */
#define SW_LOG_INFO				0
#define SW_LOG_WARNING			1
#define SW_LOG_ERROR				2
#define SW_LOG_FATAL_ERROR		3


#define sw_log	_sw_log_header(__FILE__, __LINE__), _sw_log
typedef void	(*sw_log_handler)(const sw_string message);

void SALT_DLL
_sw_log_header(
			sw_const_string	file,
			int					line);

void SALT_DLL
_sw_log(
			int					level,
			sw_const_string	function, 
			sw_const_string	format, 
			...);

void SALT_DLL
sw_set_log_handler(
			sw_log_handler	handler);

/*
 * debug is used for messages that can be turned off at compile time
 */
#if defined(NDEBUG)
#  define sw_debug 1 ? ((void) 0) : (void)
#else
#  define sw_debug _sw_debug_header(__FILE__, __LINE__), _sw_debug
void SALT_DLL
_sw_debug_header(
			sw_const_string	file,
			int					line);

void SALT_DLL
_sw_debug(
			sw_const_string	format,
			...);
#endif


void SALT_DLL
sw_debug_enable(
				sw_bool val);


#ifdef __cplusplus
}
#endif 


#endif
