#ifndef _sw_salt_h
#define _sw_salt_h

/*
 * Copyright 2003 Swampwolf Technologies, Inc. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY SWAMPWOLF TECHNOLOGIES, INC. ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Swampwolf Technologies, Inc.
 */

#include <salt/vtypes.h>
#include <salt/vtime.h>
#include <salt/dll.h>


#ifdef __cplusplus
extern "C"
{
#endif 


typedef enum _sw_context
{
	SW_CONTEXT_CONSOLE,
	SW_CONTEXT_GUI
} sw_context;


typedef enum _sw_select_event
{
   SW_SELECT_READ		=	0x1,
   SW_SELECT_WRITE	=	0x2,
   SW_SELECT_OOB		=	0x4
} sw_select_event;


typedef enum _sw_timer_type
{
   SW_ONESHOT_TIMER	=	0x1,
   SW_WALL_TIMER		=	0x2
} sw_timer_type;


/*
 * User commands start at 100 and above.  Any command lower than 100 is
 * assumed to be a salt system command.
 */
#define SW_SALT_USER_COMMAND	100


struct							_sw_salt;
typedef struct _sw_salt	*	sw_salt;
struct							_sw_address;
typedef sw_opaque				sw_select_handler;
typedef sw_result 			(*sw_select_handler_cb)(
													sw_select_handler	handler,
													sw_salt			salt,
													sw_fd					fd,
													sw_select_event	events,
													sw_opaque			extra);

typedef sw_opaque				sw_timer_handler;
typedef sw_ulong				sw_timer_id;
typedef sw_result				(*sw_timer_handler_cb)(
													sw_timer_handler	handler,
													sw_salt			salt,
													sw_timer_id			timer_id,
													sw_time				timeout,
													sw_opaque			extra);


typedef sw_opaque				sw_command_handler;
typedef sw_ulong				sw_command;
typedef sw_result				(*sw_command_handler_cb)(
													sw_command_handler	handler,
													sw_salt					salt,
													sw_command				command,
													sw_opaque				data,
													sw_opaque				extra);


sw_result SALT_DLL
sw_salt_init(
				sw_salt		*	self,
				sw_context		context,
				int				argc,
				char			**	argv);
                                 
                                 
sw_result SALT_DLL
sw_salt_fina(
				sw_salt	self);

                                                                  
sw_result SALT_DLL
sw_salt_select(
				sw_salt				self,
				sw_fd						fd,
				sw_select_event		events,
				sw_select_handler		handler,
				sw_select_handler_cb	cb,
				sw_opaque				extra);
                                 
                                 
sw_result SALT_DLL
sw_salt_register_timer(
				sw_salt					self,
				sw_timer_type			type,
				sw_time					timeout,
				sw_timer_handler		handler,
				sw_timer_handler_cb	cb,
				sw_opaque				extra,
				sw_timer_id			*	timer_id);
                                 
                                 
sw_result SALT_DLL
sw_salt_unregister_timer(
				sw_salt		self,
				sw_timer_id		timer_id);


sw_result SALT_DLL
sw_salt_send_command(
				sw_salt		self,
				sw_command	command,
				sw_opaque	data);


sw_result SALT_DLL
sw_salt_register_command_handler(
				sw_salt						self,
				sw_command					command,
				sw_command_handler		handler,
				sw_command_handler_cb	cb,
				sw_opaque					extra);


sw_result SALT_DLL
sw_salt_unregister_command_handler(
				sw_salt					self,
				sw_command				command);


sw_result SALT_DLL
sw_salt_run(
				sw_salt	self);


sw_result SALT_DLL
sw_salt_stop_run(
				sw_salt	self);
            

#ifdef __cplusplus
}
#endif


#endif
