#ifndef _salt_vstdlib_h
#define _saltsw_vstdlib_h

/*
 * Copyright 2003 Swampwolf Technologies, Inc. All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *
 *		1. Redistributions of source code must retain the above copyright notice,
 *		   this list of conditions and the following disclaimer.   
 *		2. Redistributions in binary form must reproduce the above copyright notice,
 *		   this list of conditions and the following disclaimer in the documentation
 *		   and/or other materials provided with the distribution.
 *
 *	THIS SOFTWARE IS PROVIDED BY SWAMPWOLF TECHNOLOGIES, INC. ``AS IS'' AND ANY
 *	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE HOWL PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 *	OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	The views and conclusions contained in the software and documentation are those
 *	of the authors and should not be interpreted as representing official policies,
 *	either expressed or implied, of Swampwolf Technologies, Inc.
 */

#include <salt/vtypes.h>
#include <salt/dll.h>

#if defined(WIN32)
#	include <winsock2.h>
#	include <windows.h>
#elif defined(__VXWORKS__)
#	include <sysLib.h>
#else
#	include <stdlib.h>
#	include <unistd.h>
#endif


#if defined(__cplusplus)
extern "C"
{
#endif



#if defined(NDEBUG)

#	define	sw_malloc(MEM,TYPE,SIZE)	((MEM = (TYPE) malloc(SIZE)) ? SW_OKAY : SW_E_MEM)
#	define	sw_realloc(MEM,TYPE,SIZE)	((MEM = (TYPE) realloc(MEM, SIZE)) ? SW_OKAY : SW_E_MEM)
#	define	sw_free(ARG)					if (ARG) free(ARG)

#else

#	define	sw_malloc(MEM,TYPE,SIZE)	((MEM = (TYPE) _sw_debug_malloc(SIZE, __FILE__, __LINE__)) ? SW_OKAY : SW_E_MEM)
#	define	sw_realloc(MEM,TYPE,SIZE)	((MEM = (TYPE) _sw_debug_realloc(MEM, SIZE, __FILE__, __LINE__)) ? SW_OKAY : SW_E_MEM)
#	define	sw_free(ARG)					if (ARG) _sw_debug_free(ARG, __FILE__, __LINE__)

SALT_DLL sw_opaque
_sw_debug_malloc(
			sw_ulong			size,
			const char	*	file,
			int				line);


SALT_DLL sw_opaque
_sw_debug_realloc(
			sw_opaque		mem,
			sw_ulong			size,
			const char	*	file,
			int				line);


SALT_DLL void
_sw_debug_free(
			sw_opaque		mem,
			const char	*	file,
			int				line);

#endif

#if defined(WIN32)

#	define sw_catnap(SECS)		Sleep(SECS * 1000)
#	define sw_mcatnap(MSECS)	Sleep(MSECS)
#	define sw_ucatnap(USECS)

#elif defined(__VXWORKS__)

#	define sw_catnap(SECS)		taskDelay(sysClkRateGet() * SECS)
#	define sw_mcatnap(MSECS)
#	define sw_ucatnap(USECS)

#else

#	define sw_catnap(SECS)		sleep(SECS)
#	define sw_mcatnap(MSECS)	usleep(MSECS * 1000)
#	define sw_ucatnap(USECS)	usleep(USECS)

#endif


#if defined(__cplusplus)
}
#endif


#endif
